-- Create a table to store your documents
create table {vector_name} (
id bigserial primary key,
content text, -- corresponds to Document.pageContent
metadata jsonb, -- corresponds to Document.metadata
embedding vector({vector_size}) -- 1536 works for OpenAI embeddings, change if needed
);
